@@ -201,11 +201,13 @@ void mosquitto__destroy(struct mosquitto *mosq)
 	if(!mosq) return;
 
 #ifdef WITH_THREADING
+#  ifdef HAVE_PTHREAD_CANCEL
 	if(mosq->threaded == mosq_ts_self && !pthread_equal(mosq->thread_id, pthread_self())){
 		pthread_cancel(mosq->thread_id);
 		pthread_join(mosq->thread_id, NULL);
 		mosq->threaded = mosq_ts_none;
 	}
+#  endif
 
 	if(mosq->id){
 		/* If mosq->id is not NULL then the client has already been initialised
