@@ -1,25 +1,14 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Test whether a valid CONNECT results in the correct CONNACK packet using an
 # SSL connection with client certificates required.
 
-import socket
-import ssl
-import sys
-import time
+from mosq_test_helper import *
 
 if sys.version < '2.7':
     print("WARNING: SSL not supported on Python 2.6")
     exit(0)
 
-import inspect, os
-# From http://stackoverflow.com/questions/279237/python-import-a-module-from-a-folder
-cmd_subfolder = os.path.realpath(os.path.abspath(os.path.join(os.path.split(inspect.getfile( inspect.currentframe() ))[0],"..")))
-if cmd_subfolder not in sys.path:
-    sys.path.insert(0, cmd_subfolder)
-
-import mosq_test
-
 def write_config(filename, port1, port2):
     with open(filename, 'w') as f:
         f.write("port %d\n" % (port2))
@@ -37,7 +26,6 @@ write_config(conf_file, port1, port2)
 rc = 1
 keepalive = 10
 connect_packet = mosq_test.gen_connect("connect-success-test", keepalive=keepalive)
-connack_packet = mosq_test.gen_connack(rc=0)
 
 broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=port2, use_conf=True)
 
@@ -61,7 +49,7 @@ finally:
     broker.wait()
     (stdo, stde) = broker.communicate()
     if rc:
-        print(stde)
+        print(stde.decode('utf-8'))
 
 exit(rc)
 
