@@ -43,6 +43,7 @@ tests = [
     (1, './02-subpub-qos1-nolocal.py'),
     (1, './02-subpub-qos1-v5.py'),
     (1, './02-subpub-qos1.py'),
+    (1, './02-subpub-qos2-1322.py'),
     (1, './02-subpub-qos2-bad-puback-1.py'),
     (1, './02-subpub-qos2-bad-puback-2.py'),
     (1, './02-subpub-qos2-bad-pubcomp.py'),
@@ -79,6 +80,7 @@ tests = [
     (1, './03-publish-qos1-no-subscribers-v5.py'),
     (1, './03-publish-qos1-retain-disabled.py'),
     (1, './03-publish-qos1.py'),
+    (1, './03-publish-qos2-max-inflight.py'),
     (1, './03-publish-qos2.py'),
 
     (1, './04-retain-check-source-persist.py'),
